__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Action.py 0.96.91.D001 2005/09/08 09:14:36 knight"

import dis
import os
import os.path
import re
import string
import sys

from SCons.Debug import logInstanceCreation
import SCons.Errors
import SCons.Util

class _Null:
    pass

_null = _Null

print_actions = 1
execute_actions = 1
print_actions_presub = 0

default_ENV = None

def rfile(n):
    try:
        return n.rfile()
    except AttributeError:
        return n

def default_exitstatfunc(s):
    return s

try:
    SET_LINENO = dis.SET_LINENO
    HAVE_ARGUMENT = dis.HAVE_ARGUMENT
except AttributeError:
    remove_set_lineno_codes = lambda x: x
else:
    def remove_set_lineno_codes(code):
        result = []
        n = len(code)
        i = 0
        while i < n:
            c = code[i]
            op = ord(c)
            if op >= HAVE_ARGUMENT:
                if op != SET_LINENO:
                    result.append(code[i:i+3])
                i = i+3
            else:
                result.append(c)
                i = i+1
        return string.join(result, '')

def _actionAppend(act1, act2):
    a1 = Action(act1)
    a2 = Action(act2)
    if a1 is None or a2 is None:
        raise TypeError, "Cannot append %s to %s" % (type(act1), type(act2))
    if isinstance(a1, ListAction):
        if isinstance(a2, ListAction):
            return ListAction(a1.list + a2.list)
        else:
            return ListAction(a1.list + [ a2 ])
    else:
        if isinstance(a2, ListAction):
            return ListAction([ a1 ] + a2.list)
        else:
            return ListAction([ a1, a2 ])

def _do_create_action(act, *args, **kw):
    if isinstance(act, ActionBase): return act
    if SCons.Util.is_List(act):
        return apply(CommandAction, (act,)+args, kw)
    if callable(act):
        try:
            gen = kw['generator']
            del kw['generator']
        except KeyError:
            gen = 0
        if gen:
            action_type = CommandGeneratorAction
        else:
            action_type = FunctionAction
        return apply(action_type, (act,)+args, kw)
    if SCons.Util.is_String(act):
        var=SCons.Util.get_environment_var(act)
        if var:
            return apply(LazyAction, (var,)+args, kw)
        commands = string.split(str(act), '\n')
        if len(commands) == 1:
            return apply(CommandAction, (commands[0],)+args, kw)
        else:
            listCmdActions = map(lambda x, args=args, kw=kw: apply(CommandAction, (x,)+args, kw), commands)
            return ListAction(listCmdActions)
    return None

def Action(act, *args, **kw):
    """A factory for action objects."""
    if SCons.Util.is_List(act):
        acts = map(lambda a, args=args, kw=kw:
                          apply(_do_create_action, (a,)+args, kw),
                   act)
        acts = filter(None, acts)
        if len(acts) == 1:
            return acts[0]
        else:
            return ListAction(acts)
    else:
        return apply(_do_create_action, (act,)+args, kw)

class ActionBase:
    if SCons.Memoize.use_memoizer:
        __metaclass__ = SCons.Memoize.Memoized_Metaclass
    def __cmp__(self, other):
        return cmp(self.__dict__, other)

    def genstring(self, target, source, env):
        return str(self)

    def __add__(self, other):
        return _actionAppend(self, other)

    def __radd__(self, other):
        return _actionAppend(other, self)

    def presub_lines(self, env):
        self.presub_env = env
        lines = string.split(str(self), '\n')
        self.presub_env = None      # don't need this any more
        return lines

    def get_executor(self, env, overrides, tlist, slist, executor_kw):
        return SCons.Executor.Executor(self, env, overrides,
                                       tlist, slist, executor_kw)

if SCons.Memoize.use_old_memoization():
    _Base = ActionBase
    class ActionBase(SCons.Memoize.Memoizer, _Base):
        def __init__(self, *args, **kw):
            apply(_Base.__init__, (self,)+args, kw)
            SCons.Memoize.Memoizer.__init__(self)
class _ActionAction(ActionBase):
    def __init__(self, strfunction=_null, presub=_null, chdir=None, exitstatfunc=None, **kw):
        if not strfunction is _null:
            self.strfunction = strfunction
        if presub is _null:
            presub = print_actions_presub
        self.presub = presub
        self.chdir = chdir
        if not exitstatfunc:
            exitstatfunc = default_exitstatfunc
        self.exitstatfunc = exitstatfunc
    def print_cmd_line(self, s, target, source, env):
        sys.stdout.write(s + "\n")
    def __call__(self, target, source, env, exitstatfunc=_null, presub=_null, show=_null,
                               execute=_null, chdir=_null):
        if not SCons.Util.is_List(target): target = [target]
        if not SCons.Util.is_List(source): source = [source]
        if exitstatfunc is _null: exitstatfunc = self.exitstatfunc
        if presub is _null:  presub = self.presub
        if show is _null:  show = print_actions
        if execute is _null:  execute = execute_actions
        if chdir is _null: chdir = self.chdir
        save_cwd = None
        if chdir:
            save_cwd = os.getcwd()
            try:
                chdir = str(chdir.abspath)
            except AttributeError:
                if not SCons.Util.is_String(chdir): chdir=str(target[0].dir)
        if presub:
            t = string.join(map(str, target), ' and ')
            l = string.join(self.presub_lines(env), '\n  ')
            out = "Building %s with action:\n  %s\n" % (t, l)
            sys.stdout.write(out)
        s = None
        if show and self.strfunction:
            s = self.strfunction(target, source, env)
            if s:
                if chdir:
                    s = ('os.chdir(%s)\n' % repr(chdir)) + s
                try:
                    get = env.get
                except AttributeError:
                    print_func = self.print_cmd_line
                else:
                    print_func = get('PRINT_CMD_LINE_FUNC')
                    if not print_func:
                        print_func = self.print_cmd_line
                print_func(s, target, source, env)
        stat = 0
        if execute:
            if chdir:
                os.chdir(chdir)
            try:
                stat = self.execute(target, source, env)
                stat = exitstatfunc(stat)
            finally:
                if save_cwd:
                    os.chdir(save_cwd)
        if s and save_cwd:
            print_func('os.chdir(%s)' % repr(save_cwd), target, source, env)
        return stat


def _string_from_cmd_list(cmd_list):
    cl = []
    for arg in map(str, cmd_list):
        if ' ' in arg or '\t' in arg:
            arg = '"' + arg + '"'
        cl.append(arg)
    return string.join(cl)

class CommandAction(_ActionAction):
    def __init__(self, cmd, cmdstr=None, *args, **kw):
        if __debug__: logInstanceCreation(self, 'Action.CommandAction')
        apply(_ActionAction.__init__, (self,)+args, kw)
        if SCons.Util.is_List(cmd):
            if filter(SCons.Util.is_List, cmd):
                raise TypeError, "CommandAction should be given only " \
                      "a single command"
        self.cmd_list = cmd
        self.cmdstr = cmdstr

    def __str__(self):
        if SCons.Util.is_List(self.cmd_list):
            return string.join(map(str, self.cmd_list), ' ')
        return str(self.cmd_list)

    def process(self, target, source, env):
        result = env.subst_list(self.cmd_list, 0, target, source)
        silent = None
        ignore = None
        while 1:
            try: c = result[0][0][0]
            except IndexError: c = None
            if c == '@': silent = 1
            elif c == '-': ignore = 1
            else: break
            result[0][0] = result[0][0][1:]
        try:
            if not result[0][0]:
                result[0] = result[0][1:]
        except IndexError:
            pass
        return result, ignore, silent

    def strfunction(self, target, source, env):
        if not self.cmdstr is None:
            c = env.subst(self.cmdstr, 0, target, source)
            if c:
                return c
        cmd_list, ignore, silent = self.process(target, source, env)
	if silent:
	    return ''
        return _string_from_cmd_list(cmd_list[0])

    def execute(self, target, source, env):
        from SCons.Util import is_String, is_List, flatten, escape_list
        try:
            shell = env['SHELL']
        except KeyError:
            raise SCons.Errors.UserError('Missing SHELL construction variable.')

        try:
            spawn = env['SPAWN']
        except KeyError:
            raise SCons.Errors.UserError('Missing SPAWN construction variable.')

        escape = env.get('ESCAPE', lambda x: x)

        try:
            ENV = env['ENV']
        except KeyError:
            global default_ENV
            if not default_ENV:
                import SCons.Environment
                default_ENV = SCons.Environment.Environment()['ENV']
            ENV = default_ENV

        for key, value in ENV.items():
            if not is_String(value):
                if is_List(value):
                    value = flatten(value)
                    ENV[key] = string.join(map(str, value), os.pathsep)
                else:
                    ENV[key] = str(value)

	cmd_list, ignore, silent = self.process(target, map(rfile, source), env)

        for cmd_line in filter(len, cmd_list):
            cmd_line = escape_list(cmd_line, escape)
            result = spawn(shell, escape, cmd_line[0], cmd_line, ENV)
            if not ignore and result:
                return result
        return 0

    def get_contents(self, target, source, env):
        cmd = self.cmd_list
        if SCons.Util.is_List(cmd):
            cmd = string.join(map(str, cmd))
        else:
            cmd = str(cmd)
        return env.subst_target_source(cmd, SCons.Util.SUBST_SIG, target, source)

class CommandGeneratorAction(ActionBase):
    def __init__(self, generator, *args, **kw):
        if __debug__: logInstanceCreation(self, 'Action.CommandGeneratorAction')
        self.generator = generator
        self.gen_kw = kw

    def _generate(self, target, source, env, for_signature):
        if not SCons.Util.is_List(target): target = [target]
        ret = self.generator(target=target, source=source, env=env, for_signature=for_signature)
        gen_cmd = apply(Action, (ret,), self.gen_kw)
        if not gen_cmd:
            raise SCons.Errors.UserError("Object returned from command generator: %s cannot be used to create an Action." % repr(ret))
        return gen_cmd

    def __str__(self):
        try:
            env = self.presub_env or {}
        except AttributeError:
            env = {}
        act = self._generate([], [], env, 1)
        return str(act)

    def genstring(self, target, source, env):
        return self._generate(target, source, env, 1).genstring(target, source, env)

    def __call__(self, target, source, env, exitstatfunc=_null, presub=_null,
                 show=_null, execute=_null, chdir=_null):
        act = self._generate(target, source, env, 0)
        return act(target, source, env, exitstatfunc, presub,
                   show, execute, chdir)

    def get_contents(self, target, source, env):
        return self._generate(target, source, env, 1).get_contents(target, source, env)

class LazyAction(CommandGeneratorAction, CommandAction):
    if SCons.Memoize.use_memoizer:
        __metaclass__ = SCons.Memoize.Memoized_Metaclass
    def __init__(self, var, *args, **kw):
        if __debug__: logInstanceCreation(self, 'Action.LazyAction')
        apply(CommandAction.__init__, (self, '$'+var)+args, kw)
        self.var = SCons.Util.to_String(var)
        self.gen_kw = kw
    def get_parent_class(self, env):
        c = env.get(self.var)
        if SCons.Util.is_String(c) and not '\n' in c:
            return CommandAction
        return CommandGeneratorAction
    def _generate_cache(self, env):
        """__cacheable__"""
        c = env.get(self.var, '')
        gen_cmd = apply(Action, (c,), self.gen_kw)
        if not gen_cmd:
            raise SCons.Errors.UserError("$%s value %s cannot be used to create an Action." % (self.var, repr(c)))
        return gen_cmd
    def _generate(self, target, source, env, for_signature):
        return self._generate_cache(env)

    def __call__(self, target, source, env, *args, **kw):
        args = (self, target, source, env) + args
        c = self.get_parent_class(env)
        return apply(c.__call__, args, kw)

    def get_contents(self, target, source, env):
        c = self.get_parent_class(env)
        return c.get_contents(self, target, source, env)
if not SCons.Memoize.has_metaclass:
    _Base = LazyAction
    class LazyAction(SCons.Memoize.Memoizer, _Base):
        def __init__(self, *args, **kw):
            SCons.Memoize.Memoizer.__init__(self)
            apply(_Base.__init__, (self,)+args, kw)
class FunctionAction(_ActionAction):
    def __init__(self, execfunction, *args, **kw):
        if __debug__: logInstanceCreation(self, 'Action.FunctionAction')
        self.execfunction = execfunction
        apply(_ActionAction.__init__, (self,)+args, kw)
        self.varlist = kw.get('varlist', [])
    def function_name(self):
        try:
            return self.execfunction.__name__
        except AttributeError:
            try:
                return self.execfunction.__class__.__name__
            except AttributeError:
                return "unknown_python_function"
    def strfunction(self, target, source, env):
        def array(a):
            def quote(s):
                return '"' + str(s) + '"'
            return '[' + string.join(map(quote, a), ", ") + ']'
        try:
            strfunc = self.execfunction.strfunction
        except AttributeError:
            pass
        else:
            if strfunc is None:
                return None
            if callable(strfunc):
                return strfunc(target, source, env)
        name = self.function_name()
        tstr = array(target)
        sstr = array(source)
        return "%s(%s, %s)" % (name, tstr, sstr)

    def __str__(self):
        name = self.function_name()
        if name == 'ActionCaller':
            return str(self.execfunction)
        return "%s(target, source, env)" % name

    def execute(self, target, source, env):
        rsources = map(rfile, source)
        try:
            result = self.execfunction(target=target, source=rsources, env=env)
        except EnvironmentError, e:
            raise SCons.Errors.BuildError(node=target, errstr=e.strerror)
        return result

    def get_contents(self, target, source, env):
        try:
            contents = str(self.execfunction.func_code.co_code)
        except AttributeError:
            try:
                contents = str(self.execfunction.__call__.im_func.func_code.co_code)
            except AttributeError:
                try:
                    gc = self.execfunction.get_contents
                except AttributeError:
                    contents = str(self.execfunction)
                else:
                    contents = gc(target, source, env)
        contents = remove_set_lineno_codes(contents)
        return contents + env.subst(string.join(map(lambda v: '${'+v+'}', self.varlist)))

class ListAction(ActionBase):
    def __init__(self, list):
        if __debug__: logInstanceCreation(self, 'Action.ListAction')
        def list_of_actions(x):
            if isinstance(x, ActionBase):
                return x
            return Action(x)
        self.list = map(list_of_actions, list)
    def genstring(self, target, source, env):
        return string.join(map(lambda a, t=target, s=source, e=env: a.genstring(t, s, e), self.list), '\n')
    def __str__(self):
        return string.join(map(str, self.list), '\n')
    def presub_lines(self, env):
        return SCons.Util.flatten(map(lambda a, env=env: a.presub_lines(env), self.list))
    def get_contents(self, target, source, env):
        return string.join(map(lambda x, t=target, s=source, e=env: x.get_contents(t, s, e), self.list), "")
    def __call__(self, target, source, env, exitstatfunc=_null, presub=_null, show=_null, execute=_null, chdir=_null):
        for act in self.list:
            stat = act(target, source, env, exitstatfunc, presub, show, execute, chdir)
            if stat: return stat
        return 0

class ActionCaller:
    def __init__(self, parent, args, kw):
        self.parent = parent
        self.args = args
        self.kw = kw
    def get_contents(self, target, source, env):
        actfunc = self.parent.actfunc
        try:
            contents = str(actfunc.func_code.co_code)
        except AttributeError:
            try:
                contents = str(actfunc.__call__.im_func.func_code.co_code)
            except AttributeError:
                contents = str(actfunc)
        contents = remove_set_lineno_codes(contents)
        return contents
    def subst(self, s, target, source, env):
        if s == '$__env__': return env
        else: return env.subst(s, 0, target, source)
    def subst_args(self, target, source, env):
        return map(lambda x, self=self, t=target, s=source, e=env: self.subst(x, t, s, e), self.args)
    def subst_kw(self, target, source, env):
        kw = {}
        for key in self.kw.keys():
            kw[key] = self.subst(self.kw[key], target, source, env)
        return kw
    def __call__(self, target, source, env):
        args = self.subst_args(target, source, env)
        kw = self.subst_kw(target, source, env)
        return apply(self.parent.actfunc, args, kw)
    def strfunction(self, target, source, env):
        args = self.subst_args(target, source, env)
        kw = self.subst_kw(target, source, env)
        return apply(self.parent.strfunc, args, kw)
    def __str__(self):
        return apply(self.parent.strfunc, self.args, self.kw)

class ActionFactory:
    def __init__(self, actfunc, strfunc):
        self.actfunc = actfunc
        self.strfunc = strfunc
    def __call__(self, *args, **kw):
        ac = ActionCaller(self, args, kw)
        action = Action(ac, strfunction=ac.strfunction)
        return action
